*--------------------------------------------------------------------*;
* Copyright (c) 2006 by The Biostatistics Center,                    *;
*               The George Washington University                     *;
* Web URL: http://www.bsc.gwu.edu/                                   *;
*--------------------------------------------------------------------*;

*******************************************************************;
   *
   *  WEI-LACHIN SAS MACRO
   *  WRITTEN IN MARCH 1998 BY JONI K. EVANS
   *  BASED ON THE FORTRAN DRIVER PROGRAM
   *     WRITTEN BY MAKUCH, MERRILL, AND ESCOBAR
   *  AND MODIFIED AT THE GWU BIOSTATISTICS CENTER
   *     BY B. SAUNDERS AND D.L. KENNY
   *
   *  Prepared under the direction of John M. Lachin
*******************************************************************;
*******************************************************************

     THE FOLLOWING SAS MACRO IMPLEMENTS THE WEI-LACHIN MULTIVARIATE
     GENERALIZATION OF THE GEHAN (GENERALIZED WILCOXON) AND
     LOG-RANK (SAVAGE) TESTS, FOR REPEATED OBSERVATIONS WITH CENSORING.
     REFERENCE: L.J. WEI AND J.M. LACHIN, JASA 79: PP. 653-661 (1984).

     VALUES OF THE MULTIVARIATE TEST STATISTICS FOR AN OMNIBUS
     ALTERNATIVE (EQUATION 2.2, P. 655) AND FOR AN ALTERNATIVE OF
     STOCHASTIC ORDERING (EQUATION 3.1, P. 656) ARE PROVIDED,
     CALCULATED USING WILCOXON OR LOG-RANK WEIGHTS, AS REQUESTED.
     UNIVARIATE TEST STATISTICS FOR THE GENERALIZED WILCOXON TEST OR
     THE GENERALIZED LOG-RANK TEST ALSO ARE GIVEN FOR EACH REPEATED
     MEASURE (OR OBSERVATION TIME).  ALL P-VALUES ARE TWO-SIDED.

     ADDITIONALLY, THE USER CAN REQUEST TESTS OF LOCATION-SHIFT
     ALTERNATIVES USING WILCOXON SCORES. THE MOST POWERFUL 1 DF
     TEST OF COMMON NON-ZERO OVERLAP AND THE NR-1 DF TEST OF
     HETEROGENEOUS OVERLAP ARE CARRIED OUT.  AN EXAMPLE OF THIS TYPE
     OF ANALYSIS IS GIVEN IN:
     P.F. THALL AND J.M. LACHIN, JASA 83:PP.339-347 (1988).

     A REVISED TEST OF STOCHASTIC ORDERING WAS PROPOSED BY JOHN M.
     LACHIN DURING THE SPRING OF 1990.  THIS TEST IS BASED ON THE
     THALL-LACHIN ESTIMATES OF THE UNIVARIATE SHIFT PARAMETERS
     ANALOGOUS TO THE MANN-WHITNEY THETAS.  IT IS BELIEVED TO BE
     A MORE POWERFUL TEST OF STOCHASTIC ORDERING IN CASES WHERE
     THERE ARE WIDE DIFFERENCES IN THE SAMPLE SIZES AVAILABLE FOR
     EACH VARIABLE OR REPEATED OBSERVATION TIME (AS IN INTERIM
     MONITORING OF GRADUALLY ACCRUING DATA).

     THE "REVISED" TEST OF STOCHASTIC ORDERING WAS RE-REVISED IN
     MAY 1991 TO WEIGHT THE ESTIMATED SHIFT AT EACH VISIT BY
     THE AVAILABLE SAMPLE SIZE. AN UNWEIGHTED TEST IS ALSO GIVEN.

     THE DATA SET SHOULD BE FORMATTED SO THAT EACH RECORD
     CONTAINS ALL THE REPEATED MEASURES FOR ONE SUBJECT, WITH THE
     VARIABLES OF INTEREST HAVING IDENTICAL NAMES PLUS A SUFFIX TO
     INDICATE THE TIME PERIOD, BEGINNING WITH 1 AND ENDING WITH THE
     NUMBER OF PERIODS (I.E., TIME1 TIME2 TIME3 TIME4, FOR FOUR (4)
     PERIODIC MEASUREMENTS). CORRESPONDING TO EACH MEASUREMENT
     SHOULD BE AN INDICATOR VARIABLE (I.E., IND1 IND2 IND3 IND4)
     WHICH HAS THE VALUE 1 (ONE) IF THE VALUE WAS OBSERVED, AND
     O (ZERO) IF THE VALUE IS MISSING OR CENSORED.

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   WARNING:  WHEN THIS PROGRAM IS USED TO ANALYZE DATA SETS IN WHICH:

                  THERE ARE MISSING VALUES
                      ***  AND  ***
           THE OBSERVED DATA INCLUDE NEGATIVE VALUES,

   THE MISSING VALUES MUST BE CODED AS A NEGATIVE NUMBER WHICH IS LESS
   THAN THE SMALLEST OBSERVATION.  IF THE MISSING DATA ARE CODED AS
   0 OR '.', THE PROGRAM WILL RETURN INCORRECT RESULTS.

   THIS WARNING DOES NOT APPLY TO CENSORED SURVIVAL (AS OPPOSED TO
   (MISSING MULTIVARIATE) OBSERVATIONS OR TO DATA IN WHICH ALL OF THE
   OBSERVATIONS ARE NON-NEGATIVE.
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

     IN ADDITION TO SUPPLYING THE INFORMATION ON THE MACRO CALL DEFINED
     BELOW, THE USER SHOULD DEFINE A MACRO VARIABLE (NR) WITH THE
     NUMBER OF OBSERVATION TIMES (OR REPEATED MEASURES), E.G. %LET NR=6

     NO OUTPUT IS PRINTED AS PART OF THIS MACRO, BUT ALL ESTIMATES AND
     TEST RESULTS ARE PLACED IN THREE (3) DATA SETS. IF LOCATION-SHIFT
     ANALYSES ARE REQUESTED, SIX (6) DATASETS ARE CREATED.

     WEI-LACHIN ANALYSIS:

       WLSIGMA:  THIS DATA SET CONTAINS THE FULL COVARIANCE MATRIX OF
                 THE WEI-LACHIN TEST STATISTICS, WITH NR RECORDS AND
                 NR VARIABLES NAMED WLSIG1 THROUGH WLSIG&NR

       WLSIGIN:  THIS DATA SET CONTAINS THE FULL INVERSE COV MATRIX OF
                 THE WEI-LACHIN TEST STATISTICS, WITH NR RECORDS AND
                 NR VARIABLES NAMED WLSIGI1 THROUGH WLSIGI&NR

       WLSTATS:  THIS DATA SET CONTAINS ALL WEI-LACHIN STATISTICS WITH
                 VARIANCES AND TEST RESULTS FOR UNIVARIATE TESTS AT
                 EACH OBSERVATION TIME PLUS THE TEST OF STOCHASTIC
                 ORDERING AND THE OMNIBUS TEST.
                 VARIABLES ARE DEFINED AS FOLLOWS:
                  CNTR    : SELECTION IDENTIFIER FOR THE RECORD
                            HAS VALUE 1 THROUGH NR
                                FOR UNIVARIATE TEST RESULTS
                            VALUE 41 IS THE TEST OF STOCHASTIC ORDERING
                            VALUE 42 IS THE OMNIBUS TEST
                  TOTN    : TOTAL COMBINED SAMPLE SIZE
                  N1      : NUMBER OF RECORDS IN GROUP 1
                              (INCLUDES CENSORED VALUES)
                  N2      : NUMBER OF RECORDS IN GROUP 2
                              (INCLUDES CENSORED VALUES)
                  D1      : NUMBER OF OBSERVED VALUES IN GROUP 1
                  D2      : NUMBER OF OBSERVED VALUES IN GROUP 2
                  WLT     : WEI-LACHIN STATISTIC
                  WLT_SIG : VARIANCE OF WEI-LACHIN STATISTIC
                  CHI_STAT: CHI-SQUARE TEST STATISTIC
                  PVAL    : P-VALUE
                  LABEL   : DESCRIPTION OF THE TEST IN THIS RECORD


     LOCATION SHIFT ANALYSIS (ABOVE DATASETS PLUS THE FOLLOWING)

       THETSIG:  THIS DATA SET CONTAINS THE FULL COVARIANCE MATRIX OF
                 THE THETA PARAMETERS, WITH NR RECORDS AND NR VARIABLES
                 NAMED THSIG1 THROUGH THSIG&NR

       THETSIGI: THIS DATA SET CONTAINS THE FULL INVERSE COVARIANCE
                 MATRIX OF THE THETA PARAMETERS, WITH NR RECORDS AND
                 NR VARIABLES NAMED THSIG1I THROUGH THSIGI&NR

       TLSTATS:  THIS DATA SET CONTAINS:
                    ALL WEI-LACHIN STATISTICS WITH VARIANCES,
                    LOCATION SHIFT (THETA) ESTIMATES WITH VARIANCES,
                    AND TEST RESULTS FOR UNIVARIATE TESTS AT EACH
                    OBSERVATION TIME PLUS SIX MULTIVARIATE TESTS.
                 VARIABLES ARE DEFINED AS FOLLOWS:
                  CNTR    : SELECTION IDENTIFIER FOR THE RECORD
                            HAS VALUE 1 THROUGH NR
                                FOR UNIVARIATE TEST RESULTS
                            VALUE 41 IS THE TEST OF STOCHASTIC ORDERING
                            VALUE 42 IS THE OMNIBUS TEST
                            VALUE 43 IS THE N-WEIGHTED STOCHASTIC
                                     ORDERING TEST OF LOCATION SHIFT
                            VALUE 44 IS THE UNWEIGHTED STOCHASTIC
                                     ORDERING TEST OF LOCATION SHIFT
                            VALUE 45 IS THE TEST OF COMMON LOCATION
                                     SHIFT (H0 VS H1)
                            VALUE 46 IS THE TEST OF HETEROGENEITY IN
                                     THE LOCATION SHIFT (H1 VS H2)
                  TOTN    : TOTAL COMBINED SAMPLE SIZE
                  N1      : NUMBER OF RECORDS IN GROUP 1
                              (INCLUDES CENSORED VALUES)
                  N2      : NUMBER OF RECORDS IN GROUP 2
                              (INCLUDES CENSORED VALUES)
                  D1      : NUMBER OF OBSERVED VALUES IN GROUP 1
                  D2      : NUMBER OF OBSERVED VALUES IN GROUP 2
                  WLT     : WEI-LACHIN STATISTIC
                  WLT_SIG : VARIANCE OF WEI-LACHIN STATISTIC
                  CHI_STAT: CHI-SQUARE TEST STATISTIC
                  PVAL    : P-VALUE
                  LABEL   : DESCRIPTION OF THE TEST IN THIS RECORD
                  PR1     : PROBABILITY THAT
                              VALUES OF GROUP 2 >= VALUES OF GROUP 1
                  PR2     : PROBABILITY THAT
                              VALUES OF GROUP 1 >= VALUES OF GROUP 2
                  THETA   : LOCATION SHIFT PARAMETER (PR1-PR2)
                  THET_SIG: VARIANCE OF THETA
                  THETA_SE: STANDARD ERROR OF THETA

   *******************************************************************;

   %MACRO WEILACH(METHOD, OPTION, DSET, GRPVAR, GRP1C, GRP2C,
                  PREF_D, PREF_C);

  **********************************************************************;
  *
  *     METHOD :  1 REQUESTS WILCOXON TESTS ONLY
  *               2 REQUESTS LOG-RANK TESTS ONLY
  *
  *     OPTION :  0 REQUESTS NO ADDITIONAL ANALYSES
  *               1 REQUESTS THALL-LACHIN ANALYSIS
  *                (OPTION 1 ONLY VALID IF METHOD = 1)
  *
  *       DSET :  NAME OF THE DATASET
  *
  *     GRPVAR :  NAME OF THE GROUP (TREATMENT) VARIABLE
  *
  *      GRP1C :  SINGLE DIGIT NUMERIC VALUES USED TO IDENTIFY THE GROUPS
  *      GRP2C :
  *
  *     PREF_D :  PREFIX FOR VARIABLES CONTAINING THE DATA VALUES
  *               (E.G., X1, X2, X3 .. X&NR)
  *
  *     PREF_C :  PREFIX FOR INDICATOR VARIABLES WITH CENSORING VALUES
  *               (E.G., I1, I2, I3 .. I&NR)
  *
  **********************************************************************;

%LET NNN1=0;
%LET NNN2=0;
%LET NT=0;


   %MACRO ERRPRT(ERRCODE);
       DATA _NULL_; FILE PRINT NOTITLES;
       PUT _PAGE_;
       PUT // @10 'EXECUTION OF WEI-LACHIN PROCEDURE WAS TERMINATED' /;
         %IF &ERRCODE=1 %THEN %DO;
            PUT @15 "METHOD (FIRST FIELD) MUST BE 1 (FOR WILCOXON) "
                    "OR 2 (FOR LOG-RANK)";
         %END;
         %ELSE %IF &ERRCODE=2 %THEN %DO;
            PUT @15 "LOCATION SHIFT ANALYSIS OPTION (SECOND FIELD) "
                    "MUST BE 0 (NO) OR 1 (YES)";
         %END;
         %ELSE %IF &ERRCODE=3 %THEN  %DO;
            PUT @15 "LOCATION SHIFT ANALYSIS OPTION (SECOND FIELD=1)";
            PUT @15 " NOT VALID FOR LOG-RANK METHOD (FIRST FIELD=2)";
         %END;
         %ELSE %IF &ERRCODE=4 %THEN  %DO;
            PUT @15 "NO USABLE OBSERVATIONS IN DATA, "
                    "VERIFY GROUP CODES";
         %END;
         %ELSE %IF &ERRCODE=5 %THEN  %DO;
            PUT @15 "NO USABLE OBSERVATIONS IN GROUP CODED &GRP1C";
         %END;
         %ELSE %IF &ERRCODE=6 %THEN  %DO;
            PUT @15 "NO USABLE OBSERVATIONS IN GROUP CODED &GRP2C";
         %END;
         %ELSE %IF &ERRCODE=7 %THEN  %DO;
            PUT @15 "GROUP VARIABLE (&GRPVAR) HAS VALUES OTHER THAN "
                    "&GRP1C OR &GRP2C";
         %END;
        ABORT;

   %MEND ERRPRT;



  %IF NOT(&METHOD=1 OR &METHOD=2) %THEN %DO; %ERRPRT(1); %END;
  %IF NOT(&OPTION=0 OR &OPTION=1) %THEN %DO; %ERRPRT(2); %END;
  %IF &METHOD=2 AND &OPTION=1 %THEN %DO; %ERRPRT(3); %END;
  RUN;

     *** COUNT TOTAL NUMBER OF OBS IN EACH GROUP ***;
  PROC FREQ DATA=&DSET NOPRINT;
    TABLE &GRPVAR/OUT=NNNN;
  RUN;

  DATA NNNN; SET NNNN;
    IF &GRPVAR=&GRP1C THEN DO;
       CALL SYMPUT('NNN1',COUNT);
    END;
    IF &GRPVAR=&GRP2C THEN DO;
       CALL SYMPUT('NNN2',COUNT);
    END;
  RUN;

    %LET CHKTOT = %EVAL(&NNN1+&NNN2);
    %IF &CHKTOT=0 %THEN %DO; %ERRPRT(4); %END;
    %IF &NNN1=0 %THEN %DO; %ERRPRT(5); %END;
    %IF &NNN2=0 %THEN %DO; %ERRPRT(6); %END;
  RUN;

  PROC SUMMARY DATA=NNNN; VAR COUNT;
  OUTPUT OUT=SUMMOBS SUM=NOBS;
  RUN;

  DATA SUMMOBS; SET SUMMOBS;
    CALL SYMPUT('NT',NOBS);
  RUN;

    %IF &CHKTOT ^= &NT %THEN %DO; %ERRPRT(7); %END;
  RUN;

  *** DELETE PRELIMINARY DATA SETS ***;
 PROC DATASETS LIBRARY=WORK;
  DELETE NNNN SUMMOBS;
 RUN;
 QUIT;

 DATA WLWORK; SET &DSET;
 RUN;

  %DO K = 1 %TO &NR;  ** UNIVARIATE ANALYSES FOR EACH TIME POINT **;

     *** COUNT NUMBER OF TIES IN RESPONSE VARIABLE
         AMONG OBSERVED AND CENSORED OBSERVATIONS
         WITHIN EACH TREATMENT GROUP ***;
   PROC SORT DATA=WLWORK;
     BY &PREF_D.&K &GRPVAR &PREF_C.&K;
   RUN;

   DATA WLWORK; SET WLWORK;
     RETAIN XCNT1&GRP1C XCNT1&GRP2C XCNT2&GRP1C XCNT2&GRP2C;
     BY &PREF_D.&K &GRPVAR &PREF_C.&K;
     SRTORD=_N_;
       IF FIRST.&PREF_D.&K THEN DO;
          XCNT2&GRP1C=0;  XCNT2&GRP2C=0;
          XCNT1&GRP1C=0;  XCNT1&GRP2C=0;
       END;
       IF &GRPVAR=&GRP1C THEN DO;
          XCNT2&GRP1C+1;
          IF &PREF_C.&K=1 THEN XCNT1&GRP1C+1;
       END;
       IF &GRPVAR=&GRP2C THEN DO;
          XCNT2&GRP2C+1;
          IF &PREF_C.&K=1 THEN XCNT1&GRP2C+1;
       END;
   RUN;

     *** SORT SO THAT THE FIRST RECORD FOR EACH TIE HAS
         THE MAX NUMBER OF TIES  ***;
   PROC SORT DATA=WLWORK;
     BY DESCENDING &PREF_D.&K DESCENDING SRTORD;
   RUN;

     *** COUNT NUMBER OF OBSERVATIONS WITH A VALUE
         GREATER THAN OR EQUAL TO THE CURRENT RECORD
         WITHIN EACH TREATMENT GROUP ***;
   DATA WLWORK; SET WLWORK;
     RETAIN GEX&GRP1C GEX&GRP2C 0;
     BY DESCENDING &PREF_D.&K DESCENDING SRTORD;
     IF FIRST.&PREF_D.&K THEN DO;
       GEX&GRP1C+XCNT2&GRP1C;
       GEX&GRP2C+XCNT2&GRP2C;
     END;
   RUN;

   DATA WLWORK; SET WLWORK;
     BY DESCENDING &PREF_D.&K DESCENDING SRTORD;
     SUM=GEX&GRP1C+GEX&GRP2C;

     IF &METHOD=1 THEN Q=SUM/&NT;      *** WILCOXON SCORES ***;
     IF &METHOD=2 THEN Q=1;            *** LOG RANK SCORES ***;

   *** NUMBER AT RISK OF FAILURE ***;
     R&GRP1C = GEX&GRP1C*&PREF_C.&K;
     R&GRP2C = GEX&GRP2C*&PREF_C.&K;

   *** EXPECTED NUMBER OF FAILURES ***;
     E&GRP1C = Q * R&GRP1C / SUM;
     E&GRP2C = Q * R&GRP2C / SUM;

     F&GRP1C.T&K = E&GRP2C;
     F&GRP2C.T&K = E&GRP1C;

     PSITMP&GRP1C = 0;
     PSITMP&GRP2C = 0;
     IF &PREF_C.&K=1 THEN DO;
        IF FIRST.&PREF_D.&K THEN DO;
          IF XCNT1&GRP1C=0 THEN PSITMP&GRP1C=0;
          ELSE
            PSITMP&GRP1C = XCNT1&GRP1C *
                           ROUND(F&GRP1C.T&K / R&GRP1C, 0.000000000001);
          IF XCNT1&GRP2C=0 THEN PSITMP&GRP2C=0;
          ELSE
            PSITMP&GRP2C = XCNT1&GRP2C *
                           ROUND(F&GRP2C.T&K / R&GRP2C, 0.000000000001);
        END;
     END;

   RUN;

     *** PSI IS THE SUM OF F/R FOR ALL OBSERVATIONS
         LESS THAN OR EQUAL TO THE CURRENT OBS ***;
   PROC SORT DATA=WLWORK;
     BY &PREF_D.&K DESCENDING SRTORD;
   RUN;

   DATA WLWORK; SET WLWORK;
      RETAIN PSI&GRP1C.T&K PSI&GRP2C.T&K 0;
         PSI&GRP1C.T&K+PSITMP&GRP1C;
         PSI&GRP2C.T&K+PSITMP&GRP2C;

     *** COUNT NUMBER OF UNCENSORED OBS IN EACH GROUP ***;
         DD&GRP1C.T&K=0; DD&GRP2C.T&K=0;

         IF &GRPVAR=&GRP1C AND &PREF_C.&K=1 THEN DD&GRP1C.T&K=1;
         IF &GRPVAR=&GRP2C AND &PREF_C.&K=1 THEN DD&GRP2C.T&K=1;

      *** THESE VARIABLES MUST BE RESET TO 0 FOR NEXT TIME PERIOD ***;
      DROP XCNT1&GRP1C XCNT1&GRP2C XCNT2&GRP1C XCNT2&GRP2C
           GEX&GRP1C GEX&GRP2C R&GRP1C R&GRP2C E&GRP1C E&GRP2C
           PSITMP&GRP1C PSITMP&GRP2C;
   RUN;

  %END;

  *** TOTAL NUMBER OF ELEMENTS IN
      TRIANGULAR FORM OF COVARIANCE MATRIX ***;
  %LET VMAT=%EVAL(&NR*(&NR+1)/2);

  *** COMPUTE ENTRIES IN COVARIANCE MATRIX ***;
   DATA WLWORK; SET WLWORK;
     ARRAY SIGM&GRP1C(&VMAT) SIG&GRP1C.E1-SIG&GRP1C.E&VMAT;
     ARRAY SIGM&GRP2C(&VMAT) SIG&GRP2C.E1-SIG&GRP2C.E&VMAT;
     ARRAY FFF&GRP1C(&NR) F&GRP1C.T1-F&GRP1C.T&NR;
     ARRAY FFF&GRP2C(&NR) F&GRP2C.T1-F&GRP2C.T&NR;
     ARRAY PSIA&GRP1C(&NR) PSI&GRP1C.T1-PSI&GRP1C.T&NR;
     ARRAY PSIA&GRP2C(&NR) PSI&GRP2C.T1-PSI&GRP2C.T&NR;
     N&GRP1C=%EVAL(&NNN1);
     N&GRP2C=%EVAL(&NNN2);
     DO K1 = 1 TO &NR;
       DO K2 = 1 TO K1;
          IPOINT=K1*(K1-1)/2 + K2;
          SIGM&GRP1C(IPOINT)=0;
          SIGM&GRP2C(IPOINT)=0;

         IF &GRPVAR=&GRP1C THEN DO;
             SIGM&GRP1C(IPOINT) =
                (FFF&GRP1C(K1) - PSIA&GRP1C(K1)) *
                  (FFF&GRP1C(K2) - PSIA&GRP1C(K2)) / N&GRP1C;
         END;
         IF &GRPVAR=&GRP2C THEN DO;
             SIGM&GRP2C(IPOINT) =
                (FFF&GRP2C(K1) - PSIA&GRP2C(K1)) *
                  (FFF&GRP2C(K2) - PSIA&GRP2C(K2)) / N&GRP2C;
         END;
       END;
     END;

     DO K3=1 TO &NR;
       IF &GRPVAR=&GRP1C THEN FFF&GRP2C(K3)=0;
       IF &GRPVAR=&GRP2C THEN FFF&GRP1C(K3)=0;
     END;
   RUN;

   PROC SUMMARY DATA=WLWORK;
     VAR F&GRP1C.T1-F&GRP1C.T&NR  F&GRP2C.T1-F&GRP2C.T&NR
         DD&GRP1C.T1-DD&GRP1C.T&NR SIG&GRP1C.E1-SIG&GRP1C.E&VMAT
         DD&GRP2C.T1-DD&GRP2C.T&NR SIG&GRP2C.E1-SIG&GRP2C.E&VMAT;
   OUTPUT OUT=SUMMWL
     SUM = EEE&GRP1C.T1-EEE&GRP1C.T&NR  EEE&GRP2C.T1-EEE&GRP2C.T&NR
           DDD&GRP1C.T1-DDD&GRP1C.T&NR SGM&GRP1C.E1-SGM&GRP1C.E&VMAT
           DDD&GRP2C.T1-DDD&GRP2C.T&NR SGM&GRP2C.E1-SGM&GRP2C.E&VMAT;
   RUN;

   PROC TRANSPOSE DATA=SUMMWL OUT=EEEE&GRP1C PREFIX=EE&GRP1C;
     VAR  EEE&GRP1C.T1-EEE&GRP1C.T&NR;
   RUN;

   PROC TRANSPOSE DATA=SUMMWL OUT=EEEE&GRP2C PREFIX=EE&GRP2C;
     VAR  EEE&GRP2C.T1-EEE&GRP2C.T&NR;
   RUN;

   PROC TRANSPOSE DATA=SUMMWL OUT=DDDD&GRP1C PREFIX=D&GRP1C;
     VAR  DDD&GRP1C.T1-DDD&GRP1C.T&NR;
   RUN;

   PROC TRANSPOSE DATA=SUMMWL OUT=DDDD&GRP2C PREFIX=D&GRP2C;
     VAR  DDD&GRP2C.T1-DDD&GRP2C.T&NR;
   RUN;


   DATA SIGALL; SET SUMMWL;
     DROP _TYPE_ EEE&GRP1C.T1-EEE&GRP1C.T&NR EEE&GRP2C.T1-EEE&GRP2C.T&NR
          _FREQ_ DDD&GRP1C.T1-DDD&GRP1C.T&NR DDD&GRP2C.T1-DDD&GRP2C.T&NR;
     N&GRP1C=%EVAL(&NNN1);
     N&GRP2C=%EVAL(&NNN2);
     ARRAY SIGMT&GRP1C( JJ ) SGM&GRP1C.E1-SGM&GRP1C.E&VMAT;
     ARRAY SIGMT&GRP2C( JJ ) SGM&GRP2C.E1-SGM&GRP2C.E&VMAT;
     ARRAY COVAR( JJ ) SIGMA1-SIGMA&VMAT;
     DO OVER COVAR;
       COVAR = (N&GRP1C*SIGMT&GRP1C + N&GRP2C*SIGMT&GRP2C)
                      / (N&GRP1C+N&GRP2C);
     END;
     DROP SGM&GRP1C.E1-SGM&GRP1C.E&VMAT  SGM&GRP2C.E1-SGM&GRP2C.E&VMAT;
   RUN;

  *** CONVERT FROM TRIANGULAR TO FULL MATRIX ***;
  DATA VARCOV; SET SIGALL;
    ARRAY COVAR(&VMAT) SIGMA1-SIGMA&VMAT;
    %DO K1=1 %TO &NR;
      %DO K2=1 %TO &K1;
         %LET E=%EVAL(&K1*(&K1-1)/2 + &K2);
         V&K1&K2=COVAR(&E);
      %END;
    %END;
  DROP SIGMA1-SIGMA&VMAT;
  RUN;


  DATA VARCOV; SET VARCOV;
  %IF &NR > 1 %THEN %DO;
    %DO K2=2 %TO &NR;
      %LET R=%EVAL(&K2-1);
      %DO K1=1 %TO &R;
         V&K1&K2=V&K2&K1;
      %END;
    %END;
  %END;
  RUN;

  *** CONVERT MATRIX FROM VECTOR FORM TO MATRIX FORM ***;
%LET DSTR=;
%LET CSTR=;
%DO I=1 %TO &NR;
  %LET VSTR&I=;
     %DO J=1 %TO &NR;
       %LET VSTR&I = %STR(&&VSTR&I V&I&J);
     %END;

   DATA ROW&I; SET VARCOV;
     KEEP &&VSTR&I;
   RUN;
   DATA ROW&I; SET ROW&I;
     %DO K=1 %TO &NR;
       COVAR&K = V&I&K;
     %END;
     SIG = V&I&I;
     DROP &&VSTR&I;
   RUN;
   %LET DSTR = %STR(&DSTR ROW&I);
   %LET CSTR = %STR(&CSTR COVAR&I);

%END;

   DATA WLSIGMA; SET &DSTR;
   RUN;

   *** COMPUTE WEI-LACHIN UNIVARIATE TEST STATISTICS ***;
   DATA WLSTATS;
     MERGE EEEE&GRP1C EEEE&GRP2C;
     CNTR=_N_;
     TOTN = %EVAL(&NT);
     N&GRP1C=%EVAL(&NNN1);
     N&GRP2C=%EVAL(&NNN2);
   RUN;
   DATA WLSTATS;
     MERGE WLSTATS DDDD&GRP1C DDDD&GRP2C WLSIGMA;
     DROP _NAME_;
     WLT = (EE&GRP1C.1 - EE&GRP2C.1) / SQRT(TOTN);
     WLT_SIG = SIG;
     CHI_STAT = WLT**2 / SIG;
     PVAL = 1- PROBCHI(CHI_STAT,1);
     LENGTH LABEL $ 35;
     LABEL = "OBSERVATION TIME # " || PUT(_N_,3.0);
     DROP SIG;
     RENAME EE&GRP1C.1=EE&GRP1C
            EE&GRP2C.1=EE&GRP2C
            D&GRP1C.1=D&GRP1C
            D&GRP2C.1=D&GRP2C;
   RUN;

   *** COMPUTE VALUES FOR MULTIVARIATE WEI-LACHIN
       TEST OF STOCHASTIC ORDERING ***;
   PROC SUMMARY DATA=WLSTATS;
      VAR WLT COVAR1-COVAR&NR;
   OUTPUT OUT=STOC
      SUM = TSUM SIGSUM1-SIGSUM&NR;
   RUN;

   DATA STOC; SET STOC;
     WLT_SIG = SUM (OF SIGSUM1-SIGSUM&NR);
     CHI_STAT = TSUM**2/WLT_SIG;
     PVAL = 1 - PROBCHI(CHI_STAT,1);
     RENAME TSUM=WLT;
     DROP _TYPE_ _FREQ_ SIGSUM1-SIGSUM&NR;
   RUN;

    *** PERFORM LOCATION-SHIFT ANALYSIS IF REQUESTED
        THESE ARE PRELIMINARY CALCULATIONS ***;
 %IF &OPTION=1 %THEN %DO;

    DATA TLSTATS; SET WLSTATS;
     PR1 = EE&GRP1C*TOTN/(D&GRP1C*D&GRP2C);
     PR2 = EE&GRP2C*TOTN/(D&GRP1C*D&GRP2C);
     THETA = PR1 - PR2;
     CAPD = (TOTN * SQRT(TOTN)) / (D&GRP1C*D&GRP2C);
     CAPDI = (D&GRP1C*D&GRP2C) / (TOTN * SQRT(TOTN));
     DSUM = D&GRP1C + D&GRP2C;
     DROP &CSTR;
    RUN;

 %END;

  %LET NR1 = %EVAL(&NR-1);

  *** ENTER IML TO DO MATRIX CALCULATIONS FOR MULTIVARIATE TESTS ***;

PROC IML;
USE WLSTATS;
   *** VECTOR OF WEI-LACHIN TEST STATISTICS IN A VECTOR ***;
  READ ALL VAR {WLT} INTO STAT;

  *** COVARIANCE MATRIX AS COLUMN VECTORS ***;
  READ ALL VAR {&CSTR};

  IDMAT = I(&NR);

  STATT=STAT`;

  %DO I=1 %TO &NR;
    IR&I = IDMAT[&I,];
  %END;

    *** CREATE SIGMA MATRIX FROM COLUMN VECTORS ***;
  SIGMA = COVAR1*IR1;
  %DO I=2 %TO &NR;
    SIGMA = SIGMA + COVAR&I*IR&I;
  %END;

  SIGINV = INV(SIGMA);  *** INVERSE OF SIGMA ***;

  *** PUT INVERSE MATRIX IN A DATA SET ***;
  CREATE WLSIGIN FROM SIGINV;
  APPEND FROM SIGINV;

  CHIOMB = STATT*SIGINV*STAT;  *** OMNIBUS TEST STATISTIC ***;

  *** PUT TEST STATISTIC IN A DATA SET ***;
  CREATE OMNIBUS FROM CHIOMB [COLNAME='CHIOMB'];
  APPEND FROM CHIOMB;


    *** PERFORM LOCATION-SHIFT ANALYSIS IF REQUESTED ***;

 %IF &OPTION=1 %THEN %DO;

  *** GET PRELIMINARY CALCUATIONS ***;
  USE TLSTATS VAR {THETA CAPD CAPDI DSUM};
  READ ALL;

  CAPN = %EVAL(&NR);

  THETAT = THETA`;   *** TRANSPOSE THETA VECTOR ***;
  DSUMT = DSUM`;

  JVEC = J(1,&NR,1);  *** UNITY ROW VECTOR ***;
  JVECT= J(&NR,1,1);  *** UNITY COLUMN VECTOR ***;

  CAPDMAT = DIAG(CAPD);
  CAPDIMAT = DIAG(CAPDI);

  *** CALULATE SIGMA MATRIX (AND INVERSE) FOR THETAS ***;
  SIGT = CAPDMAT * SIGMA * CAPDMAT;
  SIGTIN = CAPDIMAT * SIGINV * CAPDIMAT;

  THVAR = J(&NR,1,0);
  %DO K=1 %TO &NR;
    THVAR[&K,1] = SIGT[&K,&K];
  %END;

  *** PUT THETA COV MATRIX IN A DATA SET ***;
  CREATE THETSIG FROM SIGT;
  APPEND FROM SIGT;

  *** PUT THETA VARIANCES IN A DATA SET ***;
  CREATE THETVAR FROM THVAR [COLNAME='THET_SIG'];
  APPEND FROM THVAR;

  *** PUT THETA COV INVERSE MATRIX IN A DATA SET ***;
  CREATE THETSIGI FROM SIGTIN;
  APPEND FROM SIGTIN;

  *** CALULATE TEST STATISTIC AND VARIANCE FOR N-WEIGHTED TEST ***;
  NWTT1 = THETAT * DSUM;
  NWTT2 = JVEC * DSUM;
  NWTSTAT = NWTT1 / NWTT2;
  NWTSIG1 = DSUMT * SIGT * DSUM;
  NWTSIG = NWTSIG1 / (NWTT2**2);

  *** CALULATE TEST STATISTIC AND VARIANCE FOR UNWEIGHTED TEST ***;
  UWTT1 = JVEC * THETA;
  UWTSTAT = UWTT1 / CAPN;
  UWTSIG1 = JVEC * SIGT * JVECT;
  UWTSIG = UWTSIG1 / (CAPN**2);


  *** CALULATE VECTOR OF WEIGHTS
      FOR OPTIMAL 1 DF TEST OF COMMON SHIFT ***;
  OPTW1 = JVEC * SIGTIN;
  OPTD1 = OPTW1 * JVECT;
  OPTDEN = J(1,&NR,OPTD1);
  OPTWGT = OPTW1 / OPTDEN;
  OPTWGTT = OPTWGT`;
  *** OPTIMAL TEST STATISTIC AND VARIANCE ***;
  OPTSTAT = OPTWGT * THETA;
  OPTVAR = OPTWGT * SIGT * OPTWGTT;


  *** PERFORM TEST OF HOMOGENEITY OF COMMON OVERLAP ***;
  L = J(&NR1,&NR,0);

  %DO I=1 %TO &NR1;
    %DO J=1 %TO &NR;
      IF &J=&I THEN L[&I,&J]=1;
      IF &J=%EVAL(&I+1) THEN L[&I,&J]=-1;
    %END;
  %END;

  LT = L`;
  LSL = L*SIGT*LT;
  LSLI = INV(LSL);

  CHIHET = THETAT*LT*LSLI*L*THETA;

  CREATE COMMOV FROM CHIHET [COLNAME='CHIHET'];
  APPEND FROM CHIHET;


  OUTMAT = J(3,2,0);
  OUTMAT[1,1] = NWTSTAT;
  OUTMAT[1,2] = NWTSIG;
  OUTMAT[2,1] = UWTSTAT;
  OUTMAT[2,2] = UWTSIG;
  OUTMAT[3,1] = OPTSTAT;
  OUTMAT[3,2] = OPTVAR;

  CREATE TESTS3 FROM OUTMAT;
  APPEND FROM OUTMAT;

 %END;

QUIT;
RUN;

   *** COMBINE STOCHASTIC ORDERING AND OMNIBUS TESTS ***;
 DATA NEWSTAT; SET STOC(IN=A) OMNIBUS(IN=B);
   IF PVAL=. THEN PVAL = 1 - PROBCHI(CHIOMB,&NR);
   IF PVAL > 1.0 THEN PVAL=1.0;
   IF PVAL < 0.00001 THEN PVAL=0.00001;
   IF B THEN CHI_STAT=CHIOMB;
   DROP CHIOMB;
   LENGTH LABEL $ 35;
   IF A THEN DO;
     LABEL='TEST OF STOCHASTIC ORDERING';
     CNTR=41;
   END;
   ELSE IF B THEN DO;
     LABEL='OMNIBUS TEST VS GENERAL ALTERNATIVE';
     CNTR=42;
   END;
 RUN;

   *** COMBINE MULTIVARIATE AND UNIVARIATE TESTS ***;
 DATA WLSTATS; SET WLSTATS NEWSTAT;
   DROP EE&GRP1C EE&GRP2C &CSTR;
   WLT = ROUND(WLT,0.00001);
   WLT_SIG = ROUND(WLT_SIG,0.00001);
   CHI_STAT = ROUND(CHI_STAT,0.00001);
   PVAL = ROUND(PVAL,0.00001);
   LABEL CNTR    = 'SELECTION IDENTIFIER'
         TOTN    = 'TOTAL COMBINED SAMPLE SIZE'
         N&grp1c = "NUMBER OF RECORDS IN GROUP &grp1c"
         N&grp2c = "NUMBER OF RECORDS IN GROUP &grp2c"
         D&grp1c = "NUMBER OF OBSERVED VALUES IN GROUP &grp1c"
         D&grp2c = "NUMBER OF OBSERVED VALUES IN GROUP &grp2c"
         WLT     = 'WEI-LACHIN STATISTIC'
         WLT_SIG = 'VARIANCE OF WEI-LACHIN STATISTIC'
         CHI_STAT= 'CHI-SQUARE TEST STATISTIC'
         PVAL    = 'P-VALUE'
         LABEL   = 'DESCRIPTION OF TEST IN THIS RECORD';
 RUN;

 DATA WLSIGMA; SET WLSIGMA;
   DROP SIG;
   %DO K = 1 %TO &NR;
     RENAME COVAR&K=WLSIG&K;
   %END;
 RUN;

 DATA WLSIGIN; SET WLSIGIN;
   %DO K = 1 %TO &NR;
     RENAME COL&K=WLSIGI&K;
   %END;
 RUN;


    *** FINISH LOCATION-SHIFT ANALYSIS IF REQUESTED ***;
 %IF &OPTION=1 %THEN %DO;

 DATA THETSIG; SET THETSIG;
   %DO K = 1 %TO &NR;
     RENAME COL&K=THSIG&K;
   %END;
 RUN;

 DATA THETSIGI; SET THETSIGI;
   %DO K = 1 %TO &NR;
     RENAME COL&K=THSIGI&K;
   %END;
 RUN;


 DATA TLSTATS; MERGE TLSTATS THETVAR;
 RUN;


 DATA TESTS3; SET TESTS3;
    CHI_STAT = COL1**2/COL2;
    PVAL = 1 - PROBCHI(CHI_STAT,1);
    RENAME COL1=THETA
           COL2=THET_SIG;
    LENGTH LABEL $ 35;
    IF _N_=1 THEN DO;
       LABEL='LOC SHIFT: N-WEIGHTED STOCH. ORDER.';
       CNTR=43;
    END;
    ELSE IF _N_=2 THEN DO;
       LABEL='LOC SHIFT: UNWEIGHTED STOCH. ORDER.';
       CNTR=44;
    END;
    ELSE IF _N_=3 THEN DO;
       LABEL='LOC SHIFT: COMMON SHIFT (H0 VS H1)';
       CNTR=45;
    END;
 RUN;

 DATA COMMOV; SET COMMOV;
    CHIHET = ROUND(CHIHET,0.00001);
    PVAL = 1 - PROBCHI(CHIHET,&NR1);
    RENAME CHIHET=CHI_STAT;
    LENGTH LABEL $ 35;
    LABEL='LOC SHIFT: HETEROGENEITY (H1 VS H2)';
    CNTR=46;
 RUN;

 DATA TESTS4; SET TESTS3 COMMOV;
   IF PVAL > 1.0 THEN PVAL=1.0;
   IF PVAL < 0.00001 THEN PVAL=0.00001;
 RUN;

 DATA NEWSTAT; SET NEWSTAT TESTS4;
 RUN;

 DATA TLSTATS; SET TLSTATS NEWSTAT;
   DROP EE&GRP1C EE&GRP2C CAPD CAPDI DSUM;
   THETA_SE = ROUND(SQRT(THET_SIG),0.00001);
   WLT = ROUND(WLT,0.00001);
   WLT_SIG = ROUND(WLT_SIG,0.00001);
   CHI_STAT = ROUND(CHI_STAT,0.00001);
   PVAL = ROUND(PVAL,0.00001);
   PR1 = ROUND(PR1,0.00001);
   PR2 = ROUND(PR2,0.00001);
   THETA = ROUND(THETA,0.00001);
   THET_SIG = ROUND(THET_SIG,0.00001);
   LABEL CNTR    = 'SELECTION IDENTIFIER'
         TOTN    = 'TOTAL COMBINED SAMPLE SIZE'
         N&grp1c = "NUMBER OF RECORDS IN GROUP &grp1c"
         N&grp2c = "NUMBER OF RECORDS IN GROUP &grp2c"
         D&grp1c = "NUMBER OF OBSERVED VALUES IN GROUP &grp1c"
         D&grp2c = "NUMBER OF OBSERVED VALUES IN GROUP &grp2c"
         WLT     = 'WEI-LACHIN STATISTIC'
         WLT_SIG = 'VARIANCE OF WEI-LACHIN STATISTIC'
         CHI_STAT= 'CHI-SQUARE TEST STATISTIC'
         PVAL    = 'P-VALUE'
         LABEL   = 'DESCRIPTION OF TEST IN THIS RECORD'
         PR1     = "PR(GROUP &grp2c >= GROUP &grp1c)"
         PR2     = "PR(GROUP &grp1c >= GROUP &grp2c)"
         THETA   = 'LOCATION SHIFT PARAMETER (PR1-PR2)'
         THET_SIG= 'VARIANCE OF THETA'
         THETA_SE= 'STANDARD ERROR OF THETA';
 RUN;

   *******************************************************************

  *** DELETE INTERMEDIATE STEP DATA SETS ***;
 PROC DATASETS LIBRARY=WORK;
  DELETE TESTS3 COMMOV TESTS4 THETVAR;
 RUN;
 QUIT;

 %END;

  *** DELETE INTERMEDIATE STEP DATA SETS ***;
 PROC DATASETS LIBRARY=WORK;
  DELETE WLWORK EEEE&GRP1C EEEE&GRP2C DDDD&GRP1C DDDD&GRP2C
         SUMMWL VARCOV &DSTR SIGALL STOC OMNIBUS NEWSTAT;
 RUN;
 QUIT;

  %MEND WEILACH;

